%{
#include "x2vnc.h"

int stack[1024];
int sp = 0;

int curx = 1;
int cury = 1; 
int hold = 0;
 
void push(int x) {
	if (sp == 1023) {
		fprintf(stderr, "Stack Overflow.\n");
		exit(-1);
	}

	stack[sp++] = x;
}

int pop() {
	if (sp == 0) {
		fprintf(stderr, "Stack Underflow.\n");
		exit(-1);
	}

	return stack[--sp];
}

%}

DIGIT [0-9]
CHAR [a-zA-Z]

%option noyywrap

%%

-?{DIGIT}+ {
	push(atoi(yytext));
}

\+ {
	int a;
	int b;

	b = pop();
	a = pop();
	push(a+b);
}

\- {
	int a;
	int b;

	b = pop();
	a = pop();
	push(a-b);
}

\* {
	int a;
	int b;

	b = pop();
	a = pop();
	push(a*b);
}

\/ {
	int a;
	int b;

	b = pop();
	a = pop();
	push(a/b);
}

stack {
	printf("%d\n", sp);
}

print {
	printf("%d\n", pop());
}

width {
	push(si.framebufferWidth);
}

height {
	push(si.framebufferHeight);
}

x {
	push(curx);
}

y {
	push(cury);
}

hold {
	hold = pop();
	
	SendPointerEvent(curx, cury, hold);
}

pointerevent {
	int x;
	int y;
	int button;

	button = pop();
	y = pop();
	x = pop();

	SendPointerEvent(x, y, button);
	vncwait(0);
	
	curx = x;
	cury = y;
	hold = button;
}

move {
	int x;
	int y;

	y = pop();
	x = pop();

	if (x < 0) x += si.framebufferWidth;
	if (y < 0) y += si.framebufferHeight;
		
	SendPointerEvent(x, y, hold);

	vncwait(0);

	curx = x;
	cury = y;
}

rel {
	cury += pop();
	curx += pop();

	SendPointerEvent(curx, cury, hold);
	vncwait(0);
}

click {
	int wait = pop();
	int button = pop();

	SendPointerEvent(curx, cury, button);
	vncwait(wait);
	SendPointerEvent(curx, cury, hold);
	vncwait(0);
}
		
wait {
	vncwait(pop());
}

quit { return 0; }

[ \t\n]

{CHAR}+ {
	fprintf(stderr, "Bad command '%s'\n", yytext);
}

%%

